<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Nicolas Hodin
 *  @copyright 2010-2016 fasterImage
 *  @license   LICENSE.txt
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Fasterimage extends Module
{
    private $sApiUrl = 'https://fasterimage.io/api/';
    public $sDomainSuffix = '.fasterimage.io';
    private $bPSAddons = false;

    public function __construct()
    {
        $this->name = 'fasterimage';
        $this->tab = 'administration';
        $this->version = '1.2.1';
        $this->author = 'fasterImage';
        $this->need_instance = 1;
        $this->is_configurable = 1;
        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->module_key = '88c637f019da01cc135a82d69df0ca1a';

        parent::__construct();

        $this->displayName = $this->l('fasterImage Image Optimizer');
        $this->description = $this->l('Optimize your images & speed up your website! Your images are automatically optimized while keeping their quality.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        require_once _PS_ROOT_DIR_.'/modules/fasterimage/libs/unirest/Unirest.php';

        if (Configuration::get('FASTERIMAGE_API_KEY') == '') {
            $this->warning = $this->l('Please activate your fasterImage account in order to optimize your images');
        }

        if (Configuration::get('FASTERIMAGE_ENABLED') == '1'
            && Configuration::get('FASTERIMAGE_ACCOUNT_VALID') == '0') {
            $this->warning = $this->l('There is a problem with your API key, please check your settings.');
        }

        $this->checkAccount();
    }

    public function getContent()
    {
        $output = '';
        //$output .= $this->displayForm();

        if (Tools::isSubmit('first_submit'.$this->name)) {
            $sApiKey = Tools::strtolower((string)Tools::getValue('FASTERIMAGE_API_KEY'));

            $bOk = true;
            if (Tools::strlen($sApiKey) == 0) {
                $output .= $this->displayError($this->l('Please enter an API key'));
                $bOk = false;
            } else {
                $aData = $this->fasterimageApiGetInfo($sApiKey);
                if ($aData === false) {
                    $output .= $this->displayError($this->l('An error occured while checking your API key, please try again or contact fasterImage support.'));
                    $bOk = false;
                } else {
                    if ($aData['status'] == 'KO') {
                        $output .= $this->displayError($this->l('This API key is not valid, please try again or contact fasterImage support.'));
                        $bOk = false;
                    }
                }
            }

            if ($bOk) {
                Configuration::updateValue('FASTERIMAGE_API_KEY', $sApiKey);
                Configuration::updateValue('FASTERIMAGE_ENABLED', '1');
                Configuration::updateValue('FASTERIMAGE_DOMAIN', $aData['domain']);
                Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
                Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID', '1');

                $output .= $this->displayConfirmation($this->l('fasterImage is now enabled! '));
            } else {
                Configuration::updateValue('FASTERIMAGE_API_KEY', '');
                Configuration::updateValue('FASTERIMAGE_ENABLED', '0');
            }
        }

        if (Tools::isSubmit('submit'.$this->name)) {
            $sApiKey = (string)Tools::getValue('FASTERIMAGE_API_KEY');

            if ($sApiKey != Configuration::get('FASTERIMAGE_API_KEY')) {
                if (Tools::strlen($sApiKey) > 0) {
                    $bOk = true;
                    if (Tools::strlen($sApiKey) == 0) {
                        $output .= $this->displayError($this->l('Please enter an API key'));
                        $bOk = false;
                    } else {
                        $aData = $this->fasterimageApiGetInfo($sApiKey);
                        if ($aData === false) {
                            $output .= $this->displayError($this->l('An error occured while checking your API key, please try again or contact fasterImage support.'));
                            $bOk = false;
                        } else {
                            if ($aData['status'] == 'KO') {
                                $output .= $this->displayError($this->l('This API key is not valid, please try again or contact fasterImage support.'));
                                $bOk = false;
                            }
                        }
                    }

                    if ($bOk) {
                        Configuration::updateValue('FASTERIMAGE_API_KEY', $sApiKey);
                        Configuration::updateValue('FASTERIMAGE_DOMAIN', $aData['domain']);
                        Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
                        Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID', '1');

                        $output .= $this->displayConfirmation($this->l('fasterImage is now enabled! '));
                        $output .= $this->displayConfirmation($this->l('Settings updated'));
                    } else {
                        Configuration::updateValue('FASTERIMAGE_API_KEY', '');
                        Configuration::updateValue('FASTERIMAGE_ENABLED', '0');
                    }
                } else {
                    Configuration::updateValue('FASTERIMAGE_API_KEY', '');
                    Configuration::updateValue('FASTERIMAGE_ENABLED', '0');
                }
            } else {
                if (Tools::getValue('FASTERIMAGE_ENABLED_yes') == '1') {
                    Configuration::updateValue('FASTERIMAGE_ENABLED', '1');
                } else {
                    Configuration::updateValue('FASTERIMAGE_ENABLED', '0');
                }
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        $this->context->smarty->assign(
            array(
                'plugin_version' => $this->version,
                'ps_addons' => $this->bPSAddons
            )
        );

        $output .= $this->display(__FILE__, 'views/templates/admin/header.tpl');

        if (Configuration::get('FASTERIMAGE_API_KEY') == '' || Configuration::get('FASTERIMAGE_API_KEY') === false) {
            $aArgs = array(
                'domain' => Tools::getHttpHost(),
                'ps_addons' => $this->bPSAddons
            );

            $this->context->smarty->assign(
                $aArgs
            );
            $output .= $this->display('fasterimage', 'views/templates/admin/welcome.tpl');
            $output .= $this->displayMiniForm();
        } else {
            $aArgs = array(
                'domain' => Tools::getHttpHost(),
                'ps_addons' => $this->bPSAddons
            );

            $aData = $this->fasterimageApiGetInfo(Configuration::get('FASTERIMAGE_API_KEY'));
            if ($aData === false) {
                $aArgs['error_api'] = '1';
            } elseif ($aData['status'] == 'KO') {
                $aArgs['error_api'] = '2';
            } else {
                $aArgs['error_api'] = '0';
                $aArgs['modules_dir'] = _MODULE_DIR_;
                Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
                Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID', '1');

                $sEnabled = Configuration::get('FASTERIMAGE_ENABLED');
                $sDomain = Configuration::get('FASTERIMAGE_DOMAIN');
                $sChecked = Configuration::get('FASTERIMAGE_ACCOUNT_VALID');

                $sFileOk = ($sEnabled == '1' && Tools::strlen($sDomain) > 0 && $sChecked == '1') ? 'ok' : 'ko';

                $aArgs['domain'] = $sDomain;
                $aArgs['enabled'] = $sFileOk;
                $aArgs['nb_images'] = $aData['nb_images'];

                $iBP = $aData['bp_economisee'];
                $sFact = '';
                if ($iBP > 0) {
                    switch (rand(0, 2)) {
                        case 0:
                            $iNbSecondes3G = $iBP / 93750;
                            $secondes = floor((($iNbSecondes3G % 86400) % 3600) % 60);
                            $minutes = floor((($iNbSecondes3G % 86400) % 3600) / 60);
                            $hours = floor(($iNbSecondes3G % 86400) / 3600);
                            $days = floor($iNbSecondes3G / 86400);
                            $sDuree3G = ($days > 0 ? $days.' '.$this->l('day').($days > 1 ? 's' : '').', ' : '').($hours > 0 ? $hours.' '.$this->l('hour').($hours > 1 ? 's' : '').' ' : '').($minutes > 0 ? $minutes.' '.$this->l('minute').($minutes > 1 ? 's' : '').' '.$this->l('and') : '').$secondes.' '.$this->l('seconds');
                            $sFact = sprintf($this->l('It\'s %s of downloading on 3G!'), $sDuree3G);
                            break;
                        case 1:
                            $iNbDisquettes = round($iBP / 1440000, 1);
                            $sFact = sprintf($this->l('It\'s %s 3,5" disk%s!'), $iNbDisquettes, ($iNbDisquettes > 1 ? 's' : ''));
                            break;
                        case 2:
                            if ($iBP < 7500000000) {
                                $iRatioWikipedia = $iBP / 7500000000 * 100;
                                $iRatioWikipedia = round($iRatioWikipedia);
                                $sFact = $this->l('It\'s').' '.$iRatioWikipedia."% du volume de Wikipedia en Français!";
                                $sFact = sprintf($this->l('It\'s %s%% of french Wikipedia!'), $iRatioWikipedia);
                            } else {
                                $iRatioWikipedia = $iBP / 7500000000;
                                $iRatioWikipedia = round($iRatioWikipedia, 2);
                                $sFact = sprintf($this->l('It\'s %s times french Wikipedia!'), str_replace('.', ',', $iRatioWikipedia));
                            }


                            break;
                    }
                }
                $aArgs['bp'] = $this->formatOctets($aData['bp_economisee']);
                $aArgs['fact'] = $sFact;

                $aArgs['quota_utilisation'] = $this->formatOctets($aData['quota_utilisation']);
                $aArgs['quota_max'] = $this->formatOctets($aData['quota_max']);
            }

            $this->context->smarty->assign(
                $aArgs
            );
            $output .= $this->display(__FILE__, 'views/templates/admin/dashboard.tpl');

            $output .= $this->displayForm();

            $output .= $this->display(__FILE__, 'views/templates/admin/help.tpl');
        }

        return $output;
    }

    public function displayMiniForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fields_form = array();
        // Init Fields form array
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('API key :'),
                    'name' => 'FASTERIMAGE_API_KEY'
                )
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = false;        // false -> remove toolbar
        $helper->toolbar_scroll = false;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'first_submit'.$this->name;

        // Load current value
        $helper->fields_value['FASTERIMAGE_API_KEY'] = Configuration::get('FASTERIMAGE_API_KEY');
        $helper->fields_value['FASTERIMAGE_ENABLED'] = Configuration::get('FASTERIMAGE_ENABLED');

        return $helper->generateForm($fields_form);
    }

    public function displayForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fields_form = array();
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('API key :'),
                    'name' => 'FASTERIMAGE_API_KEY',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'checkbox',
                    'name' => 'FASTERIMAGE_ENABLED',
                    'label' => $this->l('Enable fasterImage optimization :'),
                    'values' => array(
                        'query' => array(
                            array(
                                'id' => 'yes',
                                'name' => '',
                                'val' => '1'
                            )
                        ),
                        'id' => 'id',
                        'name' => 'name'
                    )
                )
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                        '&token='.Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['FASTERIMAGE_API_KEY'] = Configuration::get('FASTERIMAGE_API_KEY');
        $helper->fields_value['FASTERIMAGE_ENABLED_yes'] = Configuration::get('FASTERIMAGE_ENABLED');

        return $helper->generateForm($fields_form);
    }

    public function fasterimageApiUpdateImages($aImages = null)
    {
        if ($aImages === null) {
            $aImages = array();
        }

        $res = $this->fasterimageApiCall('images/update', array('images' => $aImages));
        if (is_array($res) && isset($res['status']) && $res['status'] == 'OK') {
            return true;
        }

        return false;
    }

    private function fasterimageApiCall($sFunction, $aParams = null, $sApiKey = false)
    {
        if ($sApiKey === false) {
            $sApiKey = Configuration::get('FASTERIMAGE_API_KEY');
        }

        if (Tools::strlen($sApiKey) > 0) {
            $sMethod = 'GET';
            switch ($sFunction) {
                case 'images/update':
                    $sMethod = 'POST';
                    break;
            }

            $sJsonData = '';
            if ($aParams !== null && is_array($aParams)) {
                $sJsonData = Tools::jsonEncode($aParams);
            }

            if (function_exists('curl_version')) {
                $ch = curl_init();

                $aHeaders = array();
                $aHeaders[] = 'User-Agent: Prestashop'._PS_VERSION_.'/fasterImage'.$this->version.'; '
                    .(Tools::usingSecureMode() ? Tools::getShopDomainSSL(true) : Tools::getShopDomain(true));
                $aHeaders[] = 'Content-type: application/json';
                $aHeaders[] = 'Authorization: '.$sApiKey;
                if ($sMethod == 'POST') {
                    $aHeaders[] = 'Content-Length: '.Tools::strlen($sJsonData);
                }

                curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($ch, CURLOPT_HTTPHEADER, $aHeaders);
                curl_setopt($ch, CURLOPT_URL, $this->sApiUrl.'v1/'.$sFunction);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                if ($sMethod == 'POST') {
                    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $sJsonData);
                }

                try {
                    $sData = curl_exec($ch);
                } catch (Exception $ex) {
                    return false;
                }
                if (!curl_errno($ch)) {
                    $oResponse = curl_getinfo($ch);
                    if ($oResponse['http_code'] == '200' || $oResponse['http_code'] == '403' || $oResponse['http_code'] == '404') {
                        $aData = Tools::jsonDecode($sData, true);
                        return $aData;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
                curl_close($ch);
            } else {
                Unirest\Request::timeout(10);
                Unirest\Request::jsonOpts(true);
                $aHeaders = array(
                    'Authorization' => $sApiKey,
                    'User-Agent' => 'Prestashop'._PS_VERSION_.'/fasterImage'.$this->version.'; '
                        .(Tools::usingSecureMode() ? Tools::getShopDomainSSL(true) : Tools::getShopDomain(true)),
                );

                if ($sMethod == 'GET') {
                    $oResp = Unirest\Request::get($this->sApiUrl . 'v1/'.$sFunction, $aHeaders);
                } else if ($sMethod == 'POST') {
                    $oResp = Unirest\Request::post($this->sApiUrl . 'v1/'.$sFunction, $aHeaders, $sJsonData);
                }

                if ($oResp->code == '200' || $oResp->code == '403' || $oResp->code == '404') {
                    return $oResp->body;
                } else {
                    return false;
                }
            }
        }

        return false;
    }

    private function fasterimageApiGetInfo($sApiKey = false)
    {
        if ($sApiKey === false) {
            $sApiKey = Configuration::get('FASTERIMAGE_API_KEY');
        }

        if (Tools::strlen($sApiKey) > 0) {
            if (function_exists('curl_version')) {
                $ch = curl_init();

                $aHeaders = array();
                $aHeaders[] = 'User-Agent: Prestashop'._PS_VERSION_.'/fasterImage'.$this->version.'; '
                    .(Tools::usingSecureMode() ? Tools::getShopDomainSSL(true) : Tools::getShopDomain(true));
                $aHeaders[] = 'Content-type: application/json';
                $aHeaders[] = 'Authorization: '.$sApiKey;

                curl_setopt($ch, CURLOPT_HTTPHEADER, $aHeaders);
                curl_setopt($ch, CURLOPT_URL, $this->sApiUrl.'domain/info');
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

                $sData = curl_exec($ch);
                if (!curl_errno($ch)) {
                    $oResponse = curl_getinfo($ch);
                    if ($oResponse['http_code'] == '200' || $oResponse['http_code'] == '403' || $oResponse['http_code'] == '404') {
                        $aData = Tools::jsonDecode($sData, true);
                        return $aData;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
                curl_close($ch);
            } else {
                Unirest\Request::timeout(10);
                Unirest\Request::jsonOpts(true);
                $aHeaders = array(
                    'Authorization' => $sApiKey,
                    'User-Agent' => 'Prestashop'._PS_VERSION_.'/fasterImage'.$this->version.'; '
                        .(Tools::usingSecureMode() ? Tools::getShopDomainSSL(true) : Tools::getShopDomain(true)),
                );

                $oResp = Unirest\Request::get($this->sApiUrl.'domain/info', $aHeaders);

                if ($oResp->code == '200' || $oResp->code == '403' || $oResp->code == '404') {
                    return $oResp->body;
                } else {
                    return false;
                }
            }
        }


        return false;
    }

    public function hookActionCategoryUpdate($params)
    {
        //Executes after Category update
        $id_category = $params['category']->id;

        $oModuleFasterImage = Module::getInstanceByName('fasterimage');
        if ($oModuleFasterImage !== false) {
            $oCategory = new Category($id_category, $this->context->language->id);

            $this->context->link->getCatImageLink($oCategory->link_rewrite, $id_category);

            $sRootDir = str_replace(_PS_ROOT_DIR_, '', _PS_CAT_IMG_DIR_);
            $aImages = array();
            foreach (array('jpg','png') as $sImageType) {
                $aImages[] = $sRootDir . $id_category . '.' . $sImageType; //Cover
                $aImages[] = $sRootDir . $id_category . '_thumb.' . $sImageType; //Thumb
                $aImages[] = $sRootDir . $id_category . '-0_thumb.' . $sImageType; //Menu Thumb 1
                $aImages[] = $sRootDir . $id_category . '-1_thumb.' . $sImageType; //Menu Thumb 2
                $aImages[] = $sRootDir . $id_category . '-2_thumb.' . $sImageType; //Menu Thumb 3

                //All sizes
                foreach (ImageType::getImagesTypes('categories') as $aImageType) {
                    $aImages[] = $sRootDir . $id_category . '-' . $aImageType['name'] . '.' . $sImageType;
                    $aImages[] = str_replace($oModuleFasterImage->sDomainSuffix, '', $this->context->link->getCatImageLink($oCategory->link_rewrite, $id_category, $aImageType['name']));
                }
            }
            $oModuleFasterImage->fasterimageApiUpdateImages($aImages);
        }
    }

    public function hookDisplayBackOfficeHeader($params)
    {
        if (Tools::getIsset('controller') && Tools::getValue('controller') == "AdminModules" && Tools::getIsset('configure')  && Tools::getValue('configure') == "fasterimage") {
            $this->context->controller->addCSS(
                Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' .
                $this->name . '/views/css/bo-fasterimage.css'
            );
        }
    }

    public function install()
    {
        if (!parent::install()
            || !$this->registerHook('displayBackOfficeHeader')
            || !$this->registerHook('actionCategoryUpdate')
            || !Configuration::updateValue('FASTERIMAGE_ENABLED', '0')
            || !Configuration::updateValue('FASTERIMAGE_API_KEY', '')) {
            return false;
        }
        return true;
    }

    public function uninstall()
    {
        if (!parent::uninstall()
            || !$this->unregisterHook('displayBackOfficeHeader')
            || !$this->unregisterHook('actionCategoryUpdate')
            || !Configuration::deleteByName('FASTERIMAGE_ENABLED')
            || !Configuration::deleteByName('FASTERIMAGE_API_KEY')
            || !Configuration::deleteByName('FASTERIMAGE_ACCOUNT_VALID_EXP')
            || !Configuration::deleteByName('FASTERIMAGE_ACCOUNT_VALID')) {
            return false;
        }

        return true;
    }

    private function checkAccount($bForceCheck = false)
    {
        $aConfig = Configuration::getMultiple(
            array('FASTERIMAGE_ACCOUNT_VALID', 'FASTERIMAGE_ACCOUNT_VALID_EXP', 'FASTERIMAGE_API_KEY', 'FASTERIMAGE_ENABLED')
        );

        if ($bForceCheck
            || (Tools::strlen($aConfig['FASTERIMAGE_API_KEY']) > 0
                && time() - $aConfig['FASTERIMAGE_ACCOUNT_VALID_EXP'] > 60 * 60 * 24 )) {
            if ($aConfig['FASTERIMAGE_ENABLED'] === '1') {
                $aData = $this->fasterimageApiGetInfo($aConfig['FASTERIMAGE_API_KEY']);

                if ($aData === false) {
                    Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
                } else {
                    if ($aData['status'] == 'KO') {
                        Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID', '0');
                        Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
                        return false;
                    } else {
                        Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID', '1');
                        Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
                        return true;
                    }
                }
            }
            Configuration::updateValue('FASTERIMAGE_ACCOUNT_VALID_EXP', time());
        }
    }

    private function formatOctets($size, $precision = 2)
    {
        if ($size == 0) {
            return '0';
        }

        $base = log($size, 1000);
        $suffixes = array('', 'ko', 'Mo', 'Go', 'To');

        return str_replace('.', ',', round(pow(1000, $base - floor($base)), $precision)).$suffixes[floor($base)];
    }
}
